@@ -251,18 +251,20 @@ fn download_postgres(
     let url = pg_config.url().expect("no url for pg_config").as_str();
     tracing::debug!(url = %url, "Fetching");
     let http_client = build_agent_for_url(url)?;
-    let http_response = http_client.get(url).call()?;
+    let mut http_response = http_client.get(url).call()?;
+    let mut buf = Vec::new();
+    let _count = http_response.body_mut().as_reader().read_to_end(&mut buf)?;
+
     let status = http_response.status();
     tracing::trace!(status_code = %status, url = %url, "Fetched");
     if status != 200 {
         return Err(eyre!(
             "Problem downloading {}:\ncode={status}\n{}",
             pg_config.url().unwrap().to_string().yellow().bold(),
-            http_response.into_string()?
+            String::from_utf8_lossy(&buf),
         ));
     }
-    let mut buf = Vec::new();
-    let _count = http_response.into_reader().read_to_end(&mut buf)?;
+
     let pgdir = untar(&buf, pgrx_home, pg_config, init)?;
     configure_postgres(pg_config, &pgdir, init)?;
     make_postgres(pg_config, &pgdir, init)?;
@@ -273,7 +275,7 @@ fn untar(bytes: &[u8], pgrxdir: &Path, pg_config: &PgConfig, init: &Init) -> eyr
     let _token = init.jobserver.get().unwrap().acquire().unwrap();
 
     let mut unpackdir = pgrxdir.to_path_buf();
-    unpackdir.push(&format!("{}_unpack", pg_config.version()?));
+    unpackdir.push(format!("{}_unpack", pg_config.version()?));
     if unpackdir.exists() {
         // delete everything at this path if it already exists
         println!("{} {}", "     Removing".bold().green(), unpackdir.display());
