@@ -13,6 +13,7 @@ use crate::profile::CargoProfile;
 use crate::CommandExecute;
 use cargo_toml::Manifest;
 use eyre::WrapErr;
+use object::read::macho::MachOFatFile32;
 use owo_colors::OwoColorize;
 use pgrx_pg_config::cargo::PgrxManifestExt;
 use pgrx_pg_config::{get_target_dir, PgConfig, Pgrx};
@@ -343,7 +344,7 @@ fn first_build(
 
     if !features_arg.trim().is_empty() {
         command.arg("--features");
-        command.arg(&features_arg);
+        command.arg(features_arg);
     }
 
     if features.no_default_features {
@@ -358,7 +359,7 @@ fn first_build(
         command.arg(arg);
     }
 
-    let command_str = format!("{:?}", command);
+    let command_str = format!("{command:?}");
     eprintln!(
         "{} for SQL generation with features `{}`",
         "    Building".bold().green(),
@@ -367,7 +368,7 @@ fn first_build(
 
     tracing::debug!(command = %command_str, "Running");
     let cargo_output =
-        command.output().wrap_err_with(|| format!("failed to spawn cargo: {}", command_str))?;
+        command.output().wrap_err_with(|| format!("failed to spawn cargo: {command_str}"))?;
     tracing::trace!(status_code = %cargo_output.status, command = %command_str, "Finished");
 
     if !cargo_output.status.success() {
@@ -387,7 +388,7 @@ fn compute_codegen(
     dot: Option<String>,
 ) -> eyre::Result<String> {
     use proc_macro2::{Ident, Span, TokenStream};
-    let lib_name_ident = Ident::new(&lib_name, Span::call_site());
+    let lib_name_ident = Ident::new(lib_name, Span::call_site());
 
     let inputs = {
         let control_file_path = control_file_path
@@ -409,7 +410,7 @@ fn compute_codegen(
         for name in symbols.iter() {
             let name_ident = Ident::new(name, Span::call_site());
             out.extend(quote::quote! {
-                extern "Rust" {
+                unsafe extern "Rust" {
                     fn #name_ident() -> ::pgrx::pgrx_sql_entity_graph::SqlGraphEntity;
                 }
                 let entity = unsafe { #name_ident() };
@@ -458,7 +459,7 @@ fn compute_codegen(
         out
     };
     Ok(quote::quote! {
-        fn main() {
+        pub fn main() {
             #inputs
             #build
             #outputs
@@ -501,7 +502,7 @@ fn second_build(
 
     if !features_arg.trim().is_empty() {
         command.arg("--features");
-        command.arg(&features_arg);
+        command.arg(features_arg);
     }
 
     if features.no_default_features {
@@ -522,7 +523,7 @@ fn second_build(
 
     command.env("PGRX_EMBED", embed_path.as_ref());
 
-    let command_str = format!("{:?}", command);
+    let command_str = format!("{command:?}");
     eprintln!(
         "{} {}, in debug mode, for SQL generation with features {}",
         "  Rebuilding".bold().green(),
@@ -532,7 +533,7 @@ fn second_build(
 
     tracing::debug!(command = %command_str, "Running");
     let cargo_output =
-        command.output().wrap_err_with(|| format!("failed to spawn cargo: {}", command_str))?;
+        command.output().wrap_err_with(|| format!("failed to spawn cargo: {command_str}"))?;
     tracing::trace!(status_code = %cargo_output.status, command = %command_str, "Finished");
 
     if !cargo_output.status.success() {
@@ -559,11 +560,10 @@ fn compute_sql(package_name: &str, manifest: &Manifest) -> eyre::Result<()> {
     });
     command.env("CARGO_PKG_VERSION", cargo_pkg_version);
 
-    let command_str = format!("{:?}", command);
+    let command_str = format!("{command:?}");
     tracing::debug!(command = %command_str, "Running");
-    let embed_output = command
-        .output()
-        .wrap_err_with(|| format!("failed to spawn pgrx_embed: {}", command_str))?;
+    let embed_output =
+        command.output().wrap_err_with(|| format!("failed to spawn pgrx_embed: {command_str}"))?;
     tracing::trace!(status_code = %embed_output.status, command = %command_str, "Finished");
 
     if !embed_output.status.success() {
@@ -594,7 +594,6 @@ fn parse_object(data: &[u8]) -> object::Result<object::File> {
 }
 
 fn slice_arch32<'a>(data: &'a [u8], arch: &str) -> Option<&'a [u8]> {
-    use object::macho::FatHeader;
     use object::read::macho::FatArch;
     use object::Architecture;
     let target = match arch {
@@ -608,8 +607,8 @@ fn slice_arch32<'a>(data: &'a [u8], arch: &str) -> Option<&'a [u8]> {
         _ => Architecture::Unknown,
     };
 
-    let candidates = FatHeader::parse_arch32(data).ok()?;
-    let architecture = candidates.iter().find(|a| a.architecture() == target)?;
+    let candidates = MachOFatFile32::parse(data).ok()?;
+    let architecture = candidates.arches().iter().find(|a| a.architecture() == target)?;
 
     architecture.data(data).ok()
 }
