@@ -324,6 +324,10 @@ impl PgConfig {
         Ok(self.run("--bindir")?.into())
     }
 
+    pub fn lib_dir(&self) -> eyre::Result<PathBuf> {
+        Ok(self.run("--libdir")?.into())
+    }
+
     pub fn postmaster_path(&self) -> eyre::Result<PathBuf> {
         let mut path = self.bin_dir()?;
         path.push("postgres");
@@ -744,7 +748,7 @@ fn does_db_exist(pg_config: &PgConfig, dbname: &str) -> eyre::Result<bool> {
         .arg(pg_config.port()?.to_string())
         .arg("template1")
         .arg("-c")
-        .arg(&format!(
+        .arg(format!(
             "select count(*) from pg_database where datname = '{}';",
             dbname.replace('\'', "''")
         ))
